'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _debug = require('debug');

var _debug2 = _interopRequireDefault(_debug);

var _fs = require('fs');

var fs = _interopRequireWildcard(_fs);

var _path = require('path');

var path = _interopRequireWildcard(_path);

var _command = require('./command');

var _command2 = _interopRequireDefault(_command);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const debug = new _debug2.default('ordre:loader:');

class Loader {
  /**
   * @param {String} root Root-directory of project
   * @param {String} dir Sub-directory with Commands
   */
  constructor(root, dir) {
    this._root = root;
    this._dir = dir;
    this._commands = [];

    this._load();
  }

  /**
   * Recurse through `this._dir` for Commands.
   * Must contain 'Command' (i.e. FooCommand.js)
   *
   * @param {String} absolutePath Absolute path of directory w/ Commands
   * @private
   */
  _load(absolutePath = path.join(this._root, this._dir)) {
    debug(`Looking for Commands in ${absolutePath}`);

    fs.readdirSync(absolutePath).forEach(file => {
      debug(`Checking ${file}`);
      if (fs.statSync(path.join(absolutePath, file)).isDirectory()) {
        this._load(path.join(absolutePath, file));

        return;
      }

      if (file.indexOf('Command') === -1) {
        return;
      }

      let temp = require(path.join(absolutePath, file));
      if (temp.__esModule) {
        temp = temp.default;
      }

      /**
       * @type {Command}
       */
      let command;

      try {
        command = new temp();
      } catch (e) {
        if (e instanceof TypeError) {
          throw new Error(`Error when trying to instantiate class for file: ${file}. Did you export the class? \n ${e.message}`);
        }

        throw e;
      }

      if (!(command instanceof _command2.default)) {
        throw new Error(`Commands must extend Ordre.Command: ${absolutePath}`);
      }

      this._commands[command.name] = command;
    });
  }

  get commands() {
    return this._commands;
  }
}
exports.default = Loader;